from bs4 import BeautifulSoup, Tag


def add_unique_dividers(html: BeautifulSoup) -> BeautifulSoup:
    """ Create unique identity of hr (line) and sup (footnote) tags """
    for count, line in enumerate(html("hr"), start=1):
        line["id"] = f"{count}"  # add unique id for each hr tag
    for count, note in enumerate(html("sup"), start=1):
        note["id"] = f"{count}"  # add unique id for each sup tag
    return html


def check_first_footnote_in_divider(html: BeautifulSoup) -> BeautifulSoup:
    """
    Deals with edge cases like 33979, 33430, 34376:
    First sup tag contains the hr tag
    The resulting annex (which begins with <hr>) prevents capture of first footnote's contents [1]
    See example problem scenario that requires checking of first footnote:
        <sup style="color: rgb(255, 0, 0);">
        <hr align="LEFT" noshade="noshade" size="1" width="60%">
        [1]
        </sup>

    """
    if not html("hr"):
        return html
    if not isinstance((footnote_tag := html("hr")[0].parent), Tag):
        return html
    if not footnote_tag.name == "sup":
        return html
    new_hr = html.new_tag("hr", id="1")  # add an initial hr tag
    footnote_tag.insert_before(new_hr)  # add prior to footnote
    old_hr = next(c for c in footnote_tag.children if c.name == "hr")
    old_hr.decompose()  # remove erroneous hr tag
    return html


def edge_case_divider(html: BeautifulSoup) -> BeautifulSoup:
    """
    Deals with edge case 34346: no HR tag between the body and the annex;
    Deals with edge case 35293: footnotes exist but no annex
    """
    # there must be two at least two dividers
    # <sup>[1]</sup>
    # there must at least two patterns
    if not (
        len(html("hr")) == 1
        and (specific_footnotes_exist := html("sup", text="[1]"))
        and len(specific_footnotes_exist) > 2
    ):
        return html
    new_hr = html.new_tag("hr", id="0")  # add an initial hr tag
    target_tag = specific_footnotes_exist[1]  # get the 2nd one
    target_tag.insert_before(new_hr)  # add prior to footnote for PostCorpus
    return html


def create_dividing_attributes(text: str) -> str:
    """Corpus is marked with unique dividers"""
    soup = BeautifulSoup(text, "html5lib")
    marked = add_unique_dividers(soup)
    problem_note = check_first_footnote_in_divider(marked)
    edge = edge_case_divider(problem_note)
    return str(edge)  # ? return raw html markup as a string
