import re
from collections import deque
from dataclasses import dataclass, field
from typing import Collection, Dict, List, Optional, Pattern

from bs4 import BeautifulSoup, Tag


@dataclass
class PostCorpus:
    """
    This dataclass accepts a text object as input, divides the same by hr tags.
    """

    text: str

    @property
    def partitions(self) -> List[str]:
        """
        For each hr tag, look for the next one and take a slice of the content between
        """
        results = []
        html = BeautifulSoup(self.text, "html5lib")
        for tag in html("hr"):
            item = self.partition(self.text, tag)
            if not item:
                continue
            results.append(item)
        return results

    def partition(self, text: str, find_start: Tag) -> Optional[str]:
        # get start
        initiator = str(find_start)
        start = text.index(initiator)

        # get end
        find_end = find_start.find_next("hr")
        if not find_end:
            return None
        terminator = str(find_end)
        end = text.index(terminator)

        # slice
        return text[start:end]


@dataclass
class Pruned:
    """
    Convert Post Corpus list into a queue
    Remove extraneous items such as notice of judgments, order of releases, and the like
    Return the resulting items
    """

    posts: List[str]

    def __post_init__(self):
        self.q = deque(self.posts)
        self.reasons = []
        # self.first: Optional[str] = list(self.q)[0] if len(self.q) else None

    @property
    def top_of_queue(self) -> Optional[str]:
        while True:
            try:  # check if there are items in the queue and remove extraneous ones
                top_of_queue_text = self.q[0]
            except IndexError:
                return None

            if self.extraneous(top_of_queue_text):
                self.q.popleft()
            else:
                return self.q[
                    0
                ]  # {"queue": self.q[0], "reasons": self.reasons}

    def extraneous(self, text: str) -> bool:
        short_slice = text[:500]
        long_slice = text[:1000]
        return (
            self.is_notice(long_slice)
            or self.is_order_of_release(short_slice)
            or self.is_attest(long_slice)
            or self.is_address(long_slice)
        )

    def is_notice(self, text: str) -> bool:
        return (
            self.is_notice_title(text)
            or self.is_notice_please(text)
            or self.notice_variant(text)
        ) and self.no_footnotes_found(text)
        #! although a `notice of judgment` pattern is found, it may be part of a footnote

    def no_footnotes_found(self, text: str) -> bool:
        """
        Is there a footnote found in the raw string?
        """
        notes = BeautifulSoup(text, "html5lib")
        return False if len(notes("sup")) else True

    def is_notice_title(self, text: str) -> bool:
        regex = (
            r"N\s*o\s*t\s*i\s*c\s*e\s*o\s*f\s*J\s*u\s*d\s*g\s*m\s*e\s*n\s*t\s*"
        )
        match = re.compile(regex, re.I)
        return self.is_match_in_text(match, text)

    def is_notice_please(self, text: str) -> bool:
        match = re.compile(r"Please\s*take\s*notice\s*that\s*on", re.I)
        return self.is_match_in_text(match, text)

    def is_order_of_release(self, text: str) -> bool:
        regex = r"O\s*r\s*d\s*e\s*r\s*o\s*f\s*R\s*e\s*l\s*e\s*a\s*s\s*e\s*"
        match = re.compile(regex, re.I)
        return self.is_match_in_text(match, text)

    def is_attest(self, text: str) -> bool:
        regex = r"I\s*attest\s*that\s*the\s*conclusion(s)?\s*in\s*the\s*above\s*(Decision|Resolution)\s*"
        match = re.compile(regex, re.I)
        return self.is_match_in_text(match, text)

    def is_address(self, text: str) -> bool:
        match = re.compile(r"Sirs\/Mesdames\:", re.I)
        return self.is_match_in_text(match, text)

    def is_match_in_text(self, pattern: Pattern, text: str) -> bool:
        """
        Determine whether the following patterns exist in source text;
        And if they do, if they exist near the top of the source text:
        1. notice of judgment
        2. order of release
        3. attestation clause
        """
        # create html object from text
        html = BeautifulSoup(text, "html5lib")
        # find pattern in html object
        if not (tag := html.find(string=pattern)):
            return False
        # convert target html to string since pattern found
        initiator = str(tag)
        # create text variant of html object to serve as base
        base = str(html)
        # get the text position of initiator string from base
        pos = base.index(initiator)
        # if base position is near top of text, it likely is a match
        if pos < 500:
            self.reasons.append(initiator)
            return True
        return False

    def notice_variant(self, raw: str) -> bool:
        """
        Problems in finding the notice
        """
        html = BeautifulSoup(raw, "html5lib")
        regex = (
            r"N\s*o\s*t\s*i\s*c\s*e\s*o\s*f\s*J\s*u\s*d\s*g\s*m\s*e\s*n\s*t\s*"
        )
        match = re.compile(regex, re.I)
        try:
            item = str(html.center.text)
            return self.is_match_in_text(match, item)

        except AttributeError:
            try:
                item = str(html.text)
                return self.is_match_in_text(match, item)

            except ValueError:
                return False

        except:
            return False
