import re
from dataclasses import dataclass
from typing import Dict, Iterable, List, Match, Optional, Pattern

import ruling_phrase
from ruling_phrase import ordered_clause, wherefore_clause


@dataclass
class Positions:
    """
    Class for keeping track of wherefore, ordered, and voting phrases in the finale
    """

    context: str  # ideally, the finale portion (1/5th) of the raw ponencia
    index: int  # the start index of the finale portion

    @property
    def wherefore(self) -> Optional[Dict]:
        """
        Return:
        1. wherefore clause
        2. wherefore clause position, factoring in offset index
        """
        if not (wf := wherefore_clause(self.context)):
            return None
        if not (phrase := self.last(wf)):  # edge cases
            return None
        return {
            "w_clause": wf,
            "w_position": self.index + phrase.start(),
        }

    @property
    def ordered_voting(self) -> Optional[Dict]:
        """
        Return:
        1. ordered clause
        2. ordered clause start position
        2. ordered clause position, factoring in offset index
        3. voting clause position, factoring in offset index
        """
        if not (ord := ordered_clause(self.context)):
            return None
        if not (phrase := self.last(ord)):  # edge cases
            return None
        return {
            "o_clause": ord,
            "o_start": phrase.start(),
            "o_position": self.index + phrase.start(),
            "v_position": self.index + phrase.end(),
        }

    def last(self, target: str) -> Optional[Match]:
        """
        1. Format the target clause properly
        2. Clause may contain characters affecting compilation of strings,
        3. e.g. parenthesis in 'WHEREFORE, judgment is hereby rendered imposing a FINE of five thousand pesos (P5,000.00)
        4. There could be several strings of target compiled pattern in the body, return last instance
        """
        # * see unresolved unescaped characters for &amp; see case 34033
        esc: str = re.escape(target)
        pattern: Pattern = re.compile(esc)
        matches: Iterable = pattern.finditer(self.context)
        items: List[Match] = list(matches)
        return items[-1] if items else None


def get_positions(text: str):
    """Finale and positions based from the finale"""
    charcount = len(text)  # character count
    get_fifth = int(charcount / 5)  # deal with modulo
    offset_index = charcount - get_fifth  # finale start
    sliced_text = text[offset_index:]
    return Positions(sliced_text, offset_index)


def mark_positions(text: Optional[str]):
    """
    Accepts raw ponencia to generate potential:
    1. Fallo (wherefore clause),
    2. Ordered clause,
    3. Ruling,
    4. Ponencia (stripped off Fallo, Ordered clauses)

    There are four possible scenarios with respect to the text:
    1. No wherefore clause and no ordered clause
    2. No wherefore clause but with an ordered clause
    3. A wherefore clause but without an ordered clause
    4. A wherefore clause and an ordered clause
    """
    if not text:
        return None

    positions = get_positions(text)

    # check wherefore (W) and ordered (O) clauses
    if not (data := positions.wherefore):

        # * no W, no O: exit with no markers
        if not (data := positions.ordered_voting):
            return None

        # * no W, with O: exit with voting position
        o_index = data.get("v_position")
        return data | {"voting": text[o_index:], "ponencia": text[:o_index]}

    # * with W, no O (R maybe)
    # with W found, get possible ruling:
    # i.e. span of text from end of W to end of O
    # ruling starts with a short marker; whose position is before W
    w_index = data.get("w_position")
    if (phrase := ruling_phrase.fetch(text)) and len(phrase) <= 500:
        if (ruling_offset := text.find(phrase)) < w_index:
            data |= {
                "ruling": text[ruling_offset:w_index],
                "ruling_marker": phrase,
                "ruling_offset": ruling_offset,
            }

    # use W index to slice original text, get possible O:
    # i.e. in between span of text W index and end of original text
    # if O cannot be found, exit
    sliced = text[w_index:]
    if not (so_ordered := Positions(sliced, w_index).ordered_voting):
        if not (wf_clause := data.get("w_clause", None)):
            return data | {"ponencia": text[:w_index]}
        return data | {
            "ponencia": text[:w_index],
            "voting": text[w_index + len(wf_clause) :],
            "fallo": wf_clause,
        }

    # * with W, with O (R maybe)
    # both W and O exist
    # update dispositive (D) and voting (V)
    # D: fallo / dispositive - span of text from W's start to O's end
    # V: voting block - span of text from O's end to end of text
    data |= so_ordered
    return data | {
        "ponencia": text[:w_index],
        "voting": text[data.get("v_position") :],
        "fallo": sliced[: data.get("o_start")],
    }
